@@ -15,23 +15,26 @@ module Network.HTTP.Download
     , download
     , redownload
     , httpJSON
+    , httpLbs
+    , httpLBS
     , parseRequest
     , parseUrlThrow
     , setGithubHeaders
+    , withResponse
     ) where
 
 import           Stack.Prelude
 import           Stack.Types.Runner
 import qualified Data.ByteString.Lazy        as L
 import           Data.Conduit                (yield)
-import           Data.Conduit.Binary         (sourceHandle)
 import qualified Data.Conduit.Binary         as CB
 import           Data.Text.Encoding.Error    (lenientDecode)
 import           Data.Text.Encoding          (decodeUtf8With)
 import           Network.HTTP.Client         (Request, Response, path, checkResponse, parseUrlThrow, parseRequest)
 import           Network.HTTP.Client.Conduit (requestHeaders)
 import           Network.HTTP.Download.Verified
-import           Network.HTTP.Simple         (httpJSON, withResponse, getResponseBody, getResponseHeaders, getResponseStatusCode,
+import           Network.HTTP.StackClient    (httpJSON, httpLbs, httpLBS, withResponse)
+import           Network.HTTP.Simple         (getResponseBody, getResponseHeaders, getResponseStatusCode,
                                               setRequestHeader)
 import           Path.IO                     (doesFileExist)
 import           System.Directory            (createDirectoryIfMissing,
@@ -44,10 +47,10 @@ import           System.FilePath             (takeDirectory, (<.>))
 -- appropriate destination.
 --
 -- Throws an exception if things go wrong
-download :: (MonadUnliftIO m, MonadLogger m, HasRunner env, MonadReader env m)
+download :: HasRunner env
          => Request
          -> Path Abs File -- ^ destination
-         -> m Bool -- ^ Was a downloaded performed (True) or did the file already exist (False)?
+         -> RIO env Bool -- ^ Was a downloaded performed (True) or did the file already exist (False)?
 download req destpath = do
     let downloadReq = DownloadRequest
             { drRequest = req
@@ -61,12 +64,12 @@ download req destpath = do
 -- | Same as 'download', but will download a file a second time if it is already present.
 --
 -- Returns 'True' if the file was downloaded, 'False' otherwise
-redownload :: (MonadUnliftIO m, MonadLogger m, HasRunner env, MonadReader env m)
+redownload :: HasRunner env
            => Request
            -> Path Abs File -- ^ destination
-           -> m Bool
+           -> RIO env Bool
 redownload req0 dest = do
-    logDebug $ "Downloading " <> decodeUtf8With lenientDecode (path req0)
+    logDebug $ "Downloading " <> display (decodeUtf8With lenientDecode (path req0))
     let destFilePath = toFilePath dest
         etagFilePath = destFilePath <.> "etag"
 
@@ -75,8 +78,7 @@ redownload req0 dest = do
       if not exists
         then return Nothing
         else liftIO $ handleIO (const $ return Nothing) $ fmap Just $
-                 withBinaryFile etagFilePath ReadMode $ \h ->
-                     runConduit $ sourceHandle h .| CB.take 512
+                 withSourceFile etagFilePath $ \src -> runConduit $ src .| CB.take 512
 
     let req1 =
             case metag of
@@ -97,10 +99,12 @@ redownload req0 dest = do
           -- force the download to happen again.
           handleIO (const $ return ()) $ removeFile etagFilePath
 
-          runConduitRes $ getResponseBody res .| CB.sinkFileCautious destFilePath
+          withSinkFileCautious destFilePath $ \sink ->
+            runConduit $ getResponseBody res .| sink
 
           forM_ (lookup "ETag" (getResponseHeaders res)) $ \e ->
-            runConduitRes $ yield e .| CB.sinkFileCautious etagFilePath
+            withSinkFileCautious etagFilePath $ \sink ->
+            runConduit $ yield e .| sink
 
           return True
         304 -> return False
@@ -112,5 +116,4 @@ instance Exception DownloadException
 
 -- | Set the user-agent request header
 setGithubHeaders :: Request -> Request
-setGithubHeaders = setRequestHeader "User-Agent" ["The Haskell Stack"]
-                 . setRequestHeader "Accept" ["application/vnd.github.v3+json"]
+setGithubHeaders = setRequestHeader "Accept" ["application/vnd.github.v3+json"]
