@@ -18,7 +18,6 @@ import qualified Data.HashSet as HashSet
 import           Data.List
 import qualified Data.Map.Strict as M
 import qualified Data.Map.Strict as Map
-import qualified Data.Text as T
 import           Path
 import           Stack.Build.Cache
 import           Stack.Constants
@@ -33,7 +32,6 @@ import           Stack.Types.PackageDump
 import           Stack.Types.PackageIdentifier
 import           Stack.Types.PackageName
 import           Stack.Types.Version
-import           System.Process.Read (EnvOverride)
 
 -- | Options for 'getInstalled'.
 data GetInstalledOpts = GetInstalledOpts
@@ -47,8 +45,7 @@ data GetInstalledOpts = GetInstalledOpts
 
 -- | Returns the new InstalledMap and all of the locally registered packages.
 getInstalled :: HasEnvConfig env
-             => EnvOverride
-             -> GetInstalledOpts
+             => GetInstalledOpts
              -> Map PackageName PackageSource -- ^ does not contain any installed information
              -> RIO env
                   ( InstalledMap
@@ -56,7 +53,7 @@ getInstalled :: HasEnvConfig env
                   , [DumpPackage () () ()] -- snapshot installed
                   , [DumpPackage () () ()] -- locally installed
                   )
-getInstalled menv opts sourceMap = do
+getInstalled opts sourceMap = do
     logDebug "Finding out which packages are already installed"
     snapDBPath <- packageDatabaseDeps
     localDBPath <- packageDatabaseLocal
@@ -67,7 +64,7 @@ getInstalled menv opts sourceMap = do
             then configInstalledCache >>= liftM Just . loadInstalledCache
             else return Nothing
 
-    let loadDatabase' = loadDatabase menv opts mcache sourceMap
+    let loadDatabase' = loadDatabase opts mcache sourceMap
 
     (installedLibs0, globalDumpPkgs) <- loadDatabase' Nothing []
     (installedLibs1, _extraInstalled) <-
@@ -118,17 +115,16 @@ getInstalled menv opts sourceMap = do
 -- that it has profiling if necessary, and that it matches the version and
 -- location needed by the SourceMap
 loadDatabase :: HasEnvConfig env
-             => EnvOverride
-             -> GetInstalledOpts
+             => GetInstalledOpts
              -> Maybe InstalledCache -- ^ if Just, profiling or haddock is required
              -> Map PackageName PackageSource -- ^ to determine which installed things we should include
              -> Maybe (InstalledPackageLocation, Path Abs Dir) -- ^ package database, Nothing for global
              -> [LoadHelper] -- ^ from parent databases
              -> RIO env ([LoadHelper], [DumpPackage () () ()])
-loadDatabase menv opts mcache sourceMap mdb lhs0 = do
+loadDatabase opts mcache sourceMap mdb lhs0 = do
     wc <- view $ actualCompilerVersionL.to whichCompiler
-    (lhs1', dps) <- ghcPkgDump menv wc (fmap snd (maybeToList mdb))
-                $ conduitDumpPackage =$ sink
+    (lhs1', dps) <- ghcPkgDump wc (fmap snd (maybeToList mdb))
+                $ conduitDumpPackage .| sink
     let ghcjsHack = wc == Ghcjs && isNothing mdb
     lhs1 <- mapMaybeM (processLoadResult mdb ghcjsHack) lhs1'
     let lhs = pruneDeps
@@ -159,54 +155,50 @@ loadDatabase menv opts mcache sourceMap mdb lhs0 = do
             _ -> CL.map (\dp -> dp { dpSymbols = False })
     mloc = fmap fst mdb
     sinkDP = conduitProfilingCache
-           =$ conduitHaddockCache
-           =$ conduitSymbolsCache
-           =$ CL.map (isAllowed opts mcache sourceMap mloc &&& toLoadHelper mloc)
-           =$ CL.consume
+           .| conduitHaddockCache
+           .| conduitSymbolsCache
+           .| CL.map (isAllowed opts mcache sourceMap mloc &&& toLoadHelper mloc)
+           .| CL.consume
     sink = getZipSink $ (,)
         <$> ZipSink sinkDP
         <*> ZipSink CL.consume
 
-processLoadResult :: MonadLogger m
+processLoadResult :: HasLogFunc env
                   => Maybe (InstalledPackageLocation, Path Abs Dir)
                   -> Bool
                   -> (Allowed, LoadHelper)
-                  -> m (Maybe LoadHelper)
+                  -> RIO env (Maybe LoadHelper)
 processLoadResult _ _ (Allowed, lh) = return (Just lh)
 processLoadResult _ True (WrongVersion actual wanted, lh)
     -- Allow some packages in the ghcjs global DB to have the wrong
     -- versions.  Treat them as wired-ins by setting deps to [].
     | fst (lhPair lh) `HashSet.member` ghcjsBootPackages = do
-        logWarn $ T.concat
-            [ "Ignoring that the GHCJS boot package \""
-            , packageNameText (fst (lhPair lh))
-            , "\" has a different version, "
-            , versionText actual
-            , ", than the resolver's wanted version, "
-            , versionText wanted
-            ]
+        logWarn $
+            "Ignoring that the GHCJS boot package \"" <>
+            display (packageNameText (fst (lhPair lh))) <>
+            "\" has a different version, " <>
+            display (versionText actual) <>
+            ", than the resolver's wanted version, " <>
+            display (versionText wanted)
         return (Just lh)
 processLoadResult mdb _ (reason, lh) = do
-    logDebug $ T.concat $
-        [ "Ignoring package "
-        , packageNameText (fst (lhPair lh))
-        ] ++
-        maybe [] (\db -> [", from ", T.pack (show db), ","]) mdb ++
-        [ " due to"
-        , case reason of
+    logDebug $
+        "Ignoring package " <>
+        display (packageNameText (fst (lhPair lh))) <>
+        maybe mempty (\db -> ", from " <> displayShow db <> ",") mdb <>
+        " due to" <>
+        case reason of
             Allowed -> " the impossible?!?!"
             NeedsProfiling -> " it needing profiling."
             NeedsHaddock -> " it needing haddocks."
             NeedsSymbols -> " it needing debugging symbols."
             UnknownPkg -> " it being unknown to the resolver / extra-deps."
-            WrongLocation mloc loc -> " wrong location: " <> T.pack (show (mloc, loc))
-            WrongVersion actual wanted -> T.concat
-                [ " wanting version "
-                , versionText wanted
-                , " instead of "
-                , versionText actual
-                ]
-        ]
+            WrongLocation mloc loc -> " wrong location: " <> displayShow (mloc, loc)
+            WrongVersion actual wanted ->
+                " wanting version " <>
+                display (versionText wanted) <>
+                " instead of " <>
+                display (versionText actual)
     return Nothing
 
 data Allowed
@@ -238,25 +230,37 @@ isAllowed opts mcache sourceMap mloc dp
     | otherwise =
         case Map.lookup name sourceMap of
             Nothing ->
-                case mloc of
-                    -- The sourceMap has nothing to say about this global
-                    -- package, so we can use it
-                    Nothing -> Allowed
-                    Just ExtraGlobal -> Allowed
-                    -- For non-global packages, don't include unknown packages.
-                    -- See:
-                    -- https://github.com/commercialhaskell/stack/issues/292
-                    Just _ -> UnknownPkg
-            Just pii
-                | not (checkLocation (piiLocation pii)) -> WrongLocation mloc (piiLocation pii)
-                | version /= piiVersion pii -> WrongVersion version (piiVersion pii)
-                | otherwise -> Allowed
+                -- If the sourceMap has nothing to say about this package,
+                -- check if it represents a sublibrary first
+                -- See: https://github.com/commercialhaskell/stack/issues/3899
+                case dpParentLibIdent dp of
+                  Just (PackageIdentifier parentLibName version') ->
+                    case Map.lookup parentLibName sourceMap of
+                      Nothing -> checkNotFound
+                      Just pii
+                        | version' == version -> checkFound pii
+                        | otherwise -> checkNotFound -- different versions
+                  Nothing -> checkNotFound
+            Just pii -> checkFound pii
   where
     PackageIdentifier name version = dpPackageIdent dp
     -- Ensure that the installed location matches where the sourceMap says it
     -- should be installed
     checkLocation Snap = mloc /= Just (InstalledTo Local) -- we can allow either global or snap
     checkLocation Local = mloc == Just (InstalledTo Local) || mloc == Just ExtraGlobal -- 'locally' installed snapshot packages can come from extra dbs
+    -- Check if a package is allowed if it is found in the sourceMap
+    checkFound pii
+      | not (checkLocation (piiLocation pii)) = WrongLocation mloc (piiLocation pii)
+      | version /= piiVersion pii = WrongVersion version (piiVersion pii)
+      | otherwise = Allowed
+    -- check if a package is allowed if it is not found in the sourceMap
+    checkNotFound = case mloc of
+      -- The sourceMap has nothing to say about this global package, so we can use it
+      Nothing -> Allowed
+      Just ExtraGlobal -> Allowed
+      -- For non-global packages, don't include unknown packages.
+      -- See: https://github.com/commercialhaskell/stack/issues/292
+      Just _ -> UnknownPkg
 
 data LoadHelper = LoadHelper
     { lhId   :: !GhcPkgId
@@ -278,7 +282,7 @@ toLoadHelper mloc dp = LoadHelper
         if name `HashSet.member` wiredInPackages
             then []
             else dpDepends dp
-    , lhPair = (name, (toPackageLocation mloc, Library ident gid (dpLicense dp)))
+    , lhPair = (name, (toPackageLocation mloc, Library ident gid (Right <$> dpLicense dp)))
     }
   where
     gid = dpGhcPkgId dp
