@@ -79,11 +79,11 @@ import           Path
 import           Path.Extra (rejectMissingDir)
 import           Path.IO
 import           Stack.Config (getLocalPackages)
-import           Stack.Fetch (withCabalLoader)
 import           Stack.PackageIndex
 import           Stack.PackageLocation
 import           Stack.Snapshot (calculatePackagePromotion)
 import           Stack.Types.Config
+import           Stack.Types.NamedComponent
 import           Stack.Types.PackageIdentifier
 import           Stack.Types.PackageName
 import           Stack.Types.Version
@@ -230,6 +230,7 @@ resolveRawTarget globals snap deps locals (ri, rt) =
     -- Helper function: check if a 'NamedComponent' matches the given 'ComponentName'
     isCompNamed :: ComponentName -> NamedComponent -> Bool
     isCompNamed _ CLib = False
+    isCompNamed t1 (CInternalLib t2) = t1 == t2
     isCompNamed t1 (CExe t2) = t1 == t2
     isCompNamed t1 (CTest t2) = t1 == t2
     isCompNamed t1 (CBench t2) = t1 == t2
@@ -340,9 +341,8 @@ resolveRawTarget globals snap deps locals (ri, rt) =
               , rrPackageType = Dependency
               }
       where
-        getLatestVersion pn = do
-            vs <- getPackageVersions pn
-            return (fmap fst (Set.maxView vs))
+        getLatestVersion pn =
+            fmap fst . Set.maxView <$> getPackageVersions pn
 
     go (RTPackageIdentifier ident@(PackageIdentifier name version))
       | Map.member name locals = return $ Left $ T.concat
@@ -413,9 +413,9 @@ data PackageType = ProjectPackage | Dependency
   deriving (Eq, Show)
 
 combineResolveResults
-  :: forall m. MonadLogger m
+  :: forall env. HasLogFunc env
   => [ResolveResult]
-  -> m ([Text], Map PackageName Target, Map PackageName (PackageLocationIndex FilePath))
+  -> RIO env ([Text], Map PackageName Target, Map PackageName (PackageLocationIndex FilePath))
 combineResolveResults results = do
     addedDeps <- fmap Map.unions $ forM results $ \result ->
       case rrAddedDep result of
@@ -509,9 +509,9 @@ parseTargets needTargets boptscli = do
 
       drops = Set.empty -- not supported to add drops
 
-  (globals', snapshots, locals') <- withCabalLoader $ \loadFromIndex -> do
+  (globals', snapshots, locals') <- do
     addedDeps' <- fmap Map.fromList $ forM (Map.toList addedDeps) $ \(name, loc) -> do
-      gpd <- parseSingleCabalFileIndex loadFromIndex root loc
+      gpd <- parseSingleCabalFileIndex root loc
       return (name, (gpd, loc, Nothing))
 
     -- Calculate a list of all of the locals, based on the project
@@ -532,7 +532,7 @@ parseTargets needTargets boptscli = do
           ]
 
     calculatePackagePromotion
-      loadFromIndex root ls0 (Map.elems allLocals)
+      root ls0 (Map.elems allLocals)
       flags hides options drops
 
   let ls = LoadedSnapshot
