@@ -24,7 +24,7 @@ module Stack.BuildPlan
     , showItems
     ) where
 
-import           Stack.Prelude
+import           Stack.Prelude hiding (Display (..))
 import qualified Data.Foldable as F
 import qualified Data.HashSet as HashSet
 import           Data.List (intercalate)
@@ -43,11 +43,13 @@ import qualified Distribution.Types.UnqualComponentName as C
 import           Distribution.System (Platform)
 import           Distribution.Text (display)
 import qualified Distribution.Version as C
+import qualified RIO
 import           Stack.Constants
 import           Stack.Package
 import           Stack.Snapshot
 import           Stack.Types.BuildPlan
 import           Stack.Types.FlagName
+import           Stack.Types.NamedComponent
 import           Stack.Types.PackageIdentifier
 import           Stack.Types.PackageName
 import           Stack.Types.Version
@@ -428,9 +430,9 @@ selectBestSnapshot
     -> RIO env (SnapshotDef, BuildPlanCheck)
 selectBestSnapshot root gpds snaps = do
     logInfo $ "Selecting the best among "
-               <> T.pack (show (NonEmpty.length snaps))
+               <> displayShow (NonEmpty.length snaps)
                <> " snapshots...\n"
-    F.foldr1 go (NonEmpty.map (getResult <=< loadResolver . ResolverSnapshot) snaps)
+    F.foldr1 go (NonEmpty.map (getResult <=< loadResolver . ResolverStackage) snaps)
     where
         go mold mnew = do
             old@(_snap, bpc) <- mold
@@ -440,7 +442,7 @@ selectBestSnapshot root gpds snaps = do
 
         getResult snap = do
             result <- checkSnapBuildPlan root gpds Nothing snap
-              -- We know that we're only dealing with ResolverSnapshot
+              -- We know that we're only dealing with ResolverStackage
               -- here, where we can rely on the global package hints.
               -- Therefore, we don't use an actual compiler. For more
               -- info, see comments on
@@ -454,16 +456,16 @@ selectBestSnapshot root gpds snaps = do
           | otherwise = (s2, r2)
 
         reportResult BuildPlanCheckOk {} snap = do
-            logInfo $ "* Matches " <> sdResolverName snap
+            logInfo $ "* Matches " <> RIO.display (sdResolverName snap)
             logInfo ""
 
         reportResult r@BuildPlanCheckPartial {} snap = do
-            logWarn $ "* Partially matches " <> sdResolverName snap
-            logWarn $ indent $ T.pack $ show r
+            logWarn $ "* Partially matches " <> RIO.display (sdResolverName snap)
+            logWarn $ RIO.display $ indent $ T.pack $ show r
 
         reportResult r@BuildPlanCheckFail {} snap = do
-            logWarn $ "* Rejected " <> sdResolverName snap
-            logWarn $ indent $ T.pack $ show r
+            logWarn $ "* Rejected " <> RIO.display (sdResolverName snap)
+            logWarn $ RIO.display $ indent $ T.pack $ show r
 
         indent t = T.unlines $ fmap ("    " <>) (T.lines t)
 
