@@ -61,9 +61,10 @@ import           Distribution.System (OS (..), Platform (..), buildPlatform, Arc
 import qualified Distribution.Text
 import           Distribution.Version (simplifyVersionRange, mkVersion')
 import           GHC.Conc (getNumProcessors)
-import           Lens.Micro (lens)
+import           Lens.Micro (lens, set)
 import           Network.HTTP.Client (parseUrlThrow)
-import           Network.HTTP.Simple (httpJSON, getResponseBody)
+import           Network.HTTP.StackClient (httpJSON)
+import           Network.HTTP.Simple (getResponseBody)
 import           Options.Applicative (Parser, strOption, long, help)
 import           Path
 import           Path.Extra (toFilePathNoTrailingSep)
@@ -75,9 +76,9 @@ import           Stack.Config.Docker
 import           Stack.Config.Nix
 import           Stack.Config.Urls
 import           Stack.Constants
-import           Stack.Fetch
 import qualified Stack.Image as Image
 import           Stack.PackageLocation
+import           Stack.PackageIndex (CabalLoader (..), HasCabalLoader (..))
 import           Stack.Snapshot
 import           Stack.Types.BuildPlan
 import           Stack.Types.Compiler
@@ -94,17 +95,17 @@ import           Stack.Types.Version
 import           System.Environment
 import           System.PosixCompat.Files (fileOwner, getFileStatus)
 import           System.PosixCompat.User (getEffectiveUserID)
-import           System.Process.Read
+import           RIO.Process
 
 -- | If deprecated path exists, use it and print a warning.
 -- Otherwise, return the new path.
 tryDeprecatedPath
-    :: (MonadIO m, MonadLogger m)
+    :: HasLogFunc env
     => Maybe T.Text -- ^ Description of file for warning (if Nothing, no deprecation warning is displayed)
-    -> (Path Abs a -> m Bool) -- ^ Test for existence
+    -> (Path Abs a -> RIO env Bool) -- ^ Test for existence
     -> Path Abs a -- ^ New path
     -> Path Abs a -- ^ Deprecated path
-    -> m (Path Abs a, Bool) -- ^ (Path to use, whether it already exists)
+    -> RIO env (Path Abs a, Bool) -- ^ (Path to use, whether it already exists)
 tryDeprecatedPath mWarningDesc exists new old = do
     newExists <- exists new
     if newExists
@@ -116,12 +117,12 @@ tryDeprecatedPath mWarningDesc exists new old = do
                     case mWarningDesc of
                         Nothing -> return ()
                         Just desc ->
-                            logWarn $ T.concat
-                                [ "Warning: Location of ", desc, " at '"
-                                , T.pack (toFilePath old)
-                                , "' is deprecated; rename it to '"
-                                , T.pack (toFilePath new)
-                                , "' instead" ]
+                            logWarn $
+                                "Warning: Location of " <> display desc <> " at '" <>
+                                fromString (toFilePath old) <>
+                                "' is deprecated; rename it to '" <>
+                                fromString (toFilePath new) <>
+                                "' instead"
                     return (old, True)
                 else return (new, False)
 
@@ -129,8 +130,8 @@ tryDeprecatedPath mWarningDesc exists new old = do
 -- If the directory already exists at the deprecated location, its location is returned.
 -- Otherwise, the new location is returned.
 getImplicitGlobalProjectDir
-    :: (MonadIO m, MonadLogger m)
-    => Config -> m (Path Abs Dir)
+    :: HasLogFunc env
+    => Config -> RIO env (Path Abs Dir)
 getImplicitGlobalProjectDir config =
     --TEST no warning printed
     liftM fst $ tryDeprecatedPath
@@ -139,7 +140,7 @@ getImplicitGlobalProjectDir config =
         (implicitGlobalProjectDir stackRoot)
         (implicitGlobalProjectDirDeprecated stackRoot)
   where
-    stackRoot = configStackRoot config
+    stackRoot = view stackRootL config
 
 -- | This is slightly more expensive than @'asks' ('bcStackYaml' '.' 'getBuildConfig')@
 -- and should only be used when no 'BuildConfig' is at hand.
@@ -155,7 +156,7 @@ getSnapshots :: HasConfig env => RIO env Snapshots
 getSnapshots = do
     latestUrlText <- askLatestSnapshotUrl
     latestUrl <- parseUrlThrow (T.unpack latestUrlText)
-    logDebug $ "Downloading snapshot versions file from " <> latestUrlText
+    logDebug $ "Downloading snapshot versions file from " <> display latestUrlText
     result <- httpJSON latestUrl
     logDebug "Done downloading and parsing snapshot versions file"
     return $ getResponseBody result
@@ -179,17 +180,17 @@ makeConcreteResolver root ar = do
                 ProjectAndConfigMonoid project _ <-
                     loadConfigYaml (parseProjectAndConfigMonoid (parent fp)) fp
                 return $ projectResolver project
-            ARLatestNightly -> return $ ResolverSnapshot $ Nightly $ snapshotsNightly snapshots
+            ARLatestNightly -> return $ ResolverStackage $ Nightly $ snapshotsNightly snapshots
             ARLatestLTSMajor x ->
                 case IntMap.lookup x $ snapshotsLts snapshots of
                     Nothing -> throwString $ "No LTS release found with major version " ++ show x
-                    Just y -> return $ ResolverSnapshot $ LTS x y
+                    Just y -> return $ ResolverStackage $ LTS x y
             ARLatestLTS
                 | IntMap.null $ snapshotsLts snapshots -> throwString "No LTS releases found"
                 | otherwise ->
                     let (x, y) = IntMap.findMax $ snapshotsLts snapshots
-                     in return $ ResolverSnapshot $ LTS x y
-    logInfo $ "Selected resolver: " <> resolverRawName r
+                     in return $ ResolverStackage $ LTS x y
+    logInfo $ "Selected resolver: " <> display (resolverRawName r)
     return r
 
 -- | Get the latest snapshot resolver available.
@@ -200,16 +201,16 @@ getLatestResolver = do
             (x,y) <- listToMaybe (reverse (IntMap.toList (snapshotsLts snapshots)))
             return (LTS x y)
         snap = fromMaybe (Nightly (snapshotsNightly snapshots)) mlts
-    return (ResolverSnapshot snap)
+    return (ResolverStackage snap)
 
 -- | Create a 'Config' value when we're not using any local
 -- configuration files (e.g., the script command)
 configNoLocalConfig
-    :: (MonadLogger m, MonadUnliftIO m, MonadThrow m, MonadReader env m, HasRunner env)
+    :: HasRunner env
     => Path Abs Dir -- ^ stack root
     -> Maybe AbstractResolver
     -> ConfigMonoid
-    -> m Config
+    -> RIO env Config
 configNoLocalConfig _ Nothing _ = throwIO NoResolverWhenUsingNoLocalConfig
 configNoLocalConfig stackRoot (Just resolver) configMonoid = do
     userConfigPath <- liftIO $ getFakeConfigPath stackRoot resolver
@@ -223,16 +224,16 @@ configNoLocalConfig stackRoot (Just resolver) configMonoid = do
 
 -- Interprets ConfigMonoid options.
 configFromConfigMonoid
-    :: (MonadLogger m, MonadUnliftIO m, MonadThrow m, MonadReader env m, HasRunner env)
+    :: HasRunner env
     => Path Abs Dir -- ^ stack root, e.g. ~/.stack
     -> Path Abs File -- ^ user config file path, e.g. ~/.stack/config.yaml
     -> Bool -- ^ allow locals?
     -> Maybe AbstractResolver
     -> Maybe (Project, Path Abs File)
     -> ConfigMonoid
-    -> m Config
+    -> RIO env Config
 configFromConfigMonoid
-    configStackRoot configUserConfigPath configAllowLocals mresolver
+    clStackRoot configUserConfigPath configAllowLocals mresolver
     mproject ConfigMonoid{..} = do
      -- If --stack-work is passed, prefer it. Otherwise, if STACK_WORK
      -- is set, use that. If neither, use the default ".stack-work"
@@ -245,9 +246,9 @@ configFromConfigMonoid
              logWarn "The latest-snapshot-url field is deprecated in favor of 'urls' configuration"
              return (urlsFromMonoid configMonoidUrls) { urlsLatestSnapshot = url }
          _ -> return (urlsFromMonoid configMonoidUrls)
-     let configConnectionCount = fromFirst 8 configMonoidConnectionCount
+     let clConnectionCount = fromFirst 8 configMonoidConnectionCount
          configHideTHLoading = fromFirst True configMonoidHideTHLoading
-         configPackageIndices = fromFirst
+         clIndices = fromFirst
             [PackageIndex
                 { indexName = IndexName "Hackage"
                 , indexLocation = "https://s3.amazonaws.com/hackage.fpcomplete.com/"
@@ -280,7 +281,7 @@ configFromConfigMonoid
          configExtraLibDirs = configMonoidExtraLibDirs
          configOverrideGccPath = getFirst configMonoidOverrideGccPath
          configOverrideHpack = maybe HpackBundled HpackCommand $ getFirst configMonoidOverrideHpack
-         
+
          -- Only place in the codebase where platform is hard-coded. In theory
          -- in the future, allow it to be configured.
          (Platform defArch defOS) = buildPlatform
@@ -296,7 +297,8 @@ configFromConfigMonoid
          configCompilerCheck = fromFirst MatchMinor configMonoidCompilerCheck
 
      case arch of
-         OtherArch unk -> logWarn $ "Warning: Unknown value for architecture setting: " <> T.pack (show unk)
+         OtherArch "aarch64" -> return ()
+         OtherArch unk -> logWarn $ "Warning: Unknown value for architecture setting: " <> displayShow unk
          _ -> return ()
 
      configPlatformVariant <- liftIO $
@@ -304,7 +306,7 @@ configFromConfigMonoid
 
      let configBuild = buildOptsFromMonoid configMonoidBuildOpts
      configDocker <-
-         dockerOptsFromMonoid (fmap fst mproject) configStackRoot mresolver configMonoidDockerOpts
+         dockerOptsFromMonoid (fmap fst mproject) clStackRoot mresolver configMonoidDockerOpts
      configNix <- nixOptsFromMonoid configMonoidNixOpts os
 
      configSystemGHC <-
@@ -321,13 +323,14 @@ configFromConfigMonoid
          throwM ManualGHCVariantSettingsAreIncompatibleWithSystemGHC
 
      rawEnv <- liftIO getEnvironment
-     pathsEnv <- augmentPathMap configMonoidExtraPath
+     pathsEnv <- either throwM return
+               $ augmentPathMap (map toFilePath configMonoidExtraPath)
                                 (Map.fromList (map (T.pack *** T.pack) rawEnv))
-     origEnv <- mkEnvOverride configPlatform pathsEnv
-     let configEnvOverride _ = return origEnv
+     origEnv <- mkProcessContext pathsEnv
+     let configProcessContextSettings _ = return origEnv
 
      configLocalProgramsBase <- case getFirst configMonoidLocalProgramsBase of
-       Nothing -> getDefaultLocalProgramsBase configStackRoot configPlatform origEnv
+       Nothing -> getDefaultLocalProgramsBase clStackRoot configPlatform origEnv
        Just path -> return path
      platformOnlyDir <- runReaderT platformOnlyRelDir (configPlatform, configPlatformVariant)
      let configLocalPrograms = configLocalProgramsBase </> platformOnlyDir
@@ -369,18 +372,22 @@ configFromConfigMonoid
          configDefaultTemplate = getFirst configMonoidDefaultTemplate
          configDumpLogs = fromFirst DumpWarningLogs configMonoidDumpLogs
          configSaveHackageCreds = fromFirst True configMonoidSaveHackageCreds
-         configIgnoreRevisionMismatch = fromFirst False configMonoidIgnoreRevisionMismatch
+         clIgnoreRevisionMismatch = fromFirst False configMonoidIgnoreRevisionMismatch
 
      configAllowDifferentUser <-
         case getFirst configMonoidAllowDifferentUser of
             Just True -> return True
             _ -> getInContainer
 
-     configPackageCache <- liftIO $ newIORef Nothing
-
      let configMaybeProject = mproject
 
-     configRunner <- view runnerL
+     configRunner' <- view runnerL
+
+     clCache <- newIORef Nothing
+     clUpdateRef <- newMVar True
+
+     let configRunner = set processContextL origEnv configRunner'
+         configCabalLoader = CabalLoader {..}
 
      return Config {..}
 
@@ -388,7 +395,7 @@ configFromConfigMonoid
 getDefaultLocalProgramsBase :: MonadThrow m
                             => Path Abs Dir
                             -> Platform
-                            -> EnvOverride
+                            -> ProcessContext
                             -> m (Path Abs Dir)
 getDefaultLocalProgramsBase configStackRoot configPlatform override =
   let
@@ -400,7 +407,7 @@ getDefaultLocalProgramsBase configStackRoot configPlatform override =
       -- mean that Windows users would manually have to move data from the old
       -- location to the new one, which is undesirable.
       Platform _ Windows ->
-        case Map.lookup "LOCALAPPDATA" $ unEnvOverride override of
+        case Map.lookup "LOCALAPPDATA" $ view envVarsL override of
           Just t ->
             case parseAbsDir $ T.unpack t of
               Nothing -> throwM $ stringException ("Failed to parse LOCALAPPDATA environment variable (expected absolute directory): " ++ show t)
@@ -415,6 +422,10 @@ data MiniConfig = MiniConfig -- TODO do we really need a whole extra data type?
     }
 instance HasConfig MiniConfig where
     configL = lens mcConfig (\x y -> x { mcConfig = y })
+instance HasProcessContext MiniConfig where
+    processContextL = configL.processContextL
+instance HasCabalLoader MiniConfig where
+    cabalLoaderL = configL.cabalLoaderL
 instance HasPlatform MiniConfig
 instance HasGHCVariant MiniConfig where
     ghcVariantL = lens mcGHCVariant (\x y -> x { mcGHCVariant = y })
@@ -532,7 +543,7 @@ loadBuildConfig mproject maresolver mcompiler = do
               ARLatestLTS -> "lts"
               ARLatestLTSMajor x -> T.pack $ "lts-" ++ show x
               ARGlobal -> "global"
-      logDebug ("Using resolver: " <> name <> " specified on command line")
+      logDebug ("Using resolver: " <> display name <> " specified on command line")
 
       -- In order to resolve custom snapshots, we need a base
       -- directory to deal with relative paths. For the case of
@@ -550,7 +561,7 @@ loadBuildConfig mproject maresolver mcompiler = do
 
     (project', stackYamlFP) <- case mproject of
       LCSProject (project, fp, _) -> do
-          forM_ (projectUserMsg project) (logWarn . T.pack)
+          forM_ (projectUserMsg project) (logWarn . fromString)
           return (project, fp)
       LCSNoConfig _ -> do
           p <- assert (isJust mresolver) (getEmptyProject mresolver)
@@ -570,12 +581,15 @@ loadBuildConfig mproject maresolver mcompiler = do
                    when (view terminalL config) $
                        case maresolver of
                            Nothing ->
-                               logDebug ("Using resolver: " <> resolverRawName (projectResolver project) <>
-                                         " from implicit global project's config file: " <> T.pack dest')
+                               logDebug $
+                                 "Using resolver: " <>
+                                 display (resolverRawName (projectResolver project)) <>
+                                 " from implicit global project's config file: " <>
+                                 fromString dest'
                            Just _ -> return ()
                    return (project, dest)
                else do
-                   logInfo ("Writing implicit global project config file to: " <> T.pack dest')
+                   logInfo ("Writing implicit global project config file to: " <> fromString dest')
                    logInfo "Note: You can change the snapshot via the resolver field there."
                    p <- getEmptyProject mresolver
                    liftIO $ do
@@ -623,11 +637,11 @@ loadBuildConfig mproject maresolver mcompiler = do
     getEmptyProject mresolver = do
       r <- case mresolver of
             Just resolver -> do
-                logInfo ("Using resolver: " <> resolverRawName resolver <> " specified on command line")
+                logInfo ("Using resolver: " <> display (resolverRawName resolver) <> " specified on command line")
                 return resolver
             Nothing -> do
                 r'' <- getLatestResolver
-                logInfo ("Using latest snapshot resolver: " <> resolverRawName r'')
+                logInfo ("Using latest snapshot resolver: " <> display (resolverRawName r''))
                 return r''
       return Project
         { projectUserMsg = Nothing
@@ -640,14 +654,14 @@ loadBuildConfig mproject maresolver mcompiler = do
         }
 
 -- | Get packages from EnvConfig, downloading and cloning as necessary.
--- If the packages have already been downloaded, this uses a cached value (
+-- If the packages have already been downloaded, this uses a cached value.
 getLocalPackages :: forall env. HasEnvConfig env => RIO env LocalPackages
 getLocalPackages = do
     cacheRef <- view $ envConfigL.to envConfigPackagesRef
     mcached <- liftIO $ readIORef cacheRef
     case mcached of
         Just cached -> return cached
-        Nothing -> withCabalLoader $ \loadFromIndex -> do
+        Nothing -> do
             root <- view projectRootL
             bc <- view buildConfigL
 
@@ -661,8 +675,8 @@ getLocalPackages = do
                               $ C.package
                               $ C.packageDescription gpd
                       in (name, (gpd, loc))
-            deps <- (map wrapGPD . concat)
-                <$> mapM (parseMultiCabalFilesIndex loadFromIndex root) (bcDependencies bc)
+            deps <- map wrapGPD . concat
+                <$> mapM (parseMultiCabalFilesIndex root) (bcDependencies bc)
 
             checkDuplicateNames $
               map (second (PLOther . lpvLoc)) packages ++
@@ -759,12 +773,6 @@ isOwnedByUser path = liftIO $ do
             fileStatus <- getFileStatus (toFilePath path)
             user <- getEffectiveUserID
             return (user == fileOwner fileStatus)
-  where
-#ifdef WINDOWS
-    osIsWindows = True
-#else
-    osIsWindows = False
-#endif
 
 -- | 'True' if we are currently running inside a Docker container.
 getInContainer :: (MonadIO m) => m Bool
@@ -777,9 +785,9 @@ getInNixShell = liftIO (isJust <$> lookupEnv inNixShellEnvVar)
 -- | Determine the extra config file locations which exist.
 --
 -- Returns most local first
-getExtraConfigs :: (MonadIO m, MonadLogger m)
+getExtraConfigs :: HasLogFunc env
                 => Path Abs File -- ^ use config path
-                -> m [Path Abs File]
+                -> RIO env [Path Abs File]
 getExtraConfigs userConfigPath = do
   defaultStackGlobalConfigPath <- getDefaultGlobalConfigPath
   liftIO $ do
@@ -797,8 +805,8 @@ getExtraConfigs userConfigPath = do
 -- | Load and parse YAML from the given config file. Throws
 -- 'ParseConfigFileException' when there's a decoding error.
 loadConfigYaml
-    :: (MonadIO m, MonadLogger m)
-    => (Value -> Yaml.Parser (WithJSONWarnings a)) -> Path Abs File -> m a
+    :: HasLogFunc env
+    => (Value -> Yaml.Parser (WithJSONWarnings a)) -> Path Abs File -> RIO env a
 loadConfigYaml parser path = do
     eres <- loadYaml parser path
     case eres of
@@ -807,8 +815,8 @@ loadConfigYaml parser path = do
 
 -- | Load and parse YAML from the given file.
 loadYaml
-    :: (MonadIO m, MonadLogger m)
-    => (Value -> Yaml.Parser (WithJSONWarnings a)) -> Path Abs File -> m (Either Yaml.ParseException a)
+    :: HasLogFunc env
+    => (Value -> Yaml.Parser (WithJSONWarnings a)) -> Path Abs File -> RIO env (Either Yaml.ParseException a)
 loadYaml parser path = do
     eres <- liftIO $ Yaml.decodeFileEither (toFilePath path)
     case eres  of
@@ -821,10 +829,10 @@ loadYaml parser path = do
                     return (Right res)
 
 -- | Get the location of the project config file, if it exists.
-getProjectConfig :: (MonadIO m, MonadThrow m, MonadLogger m)
+getProjectConfig :: HasLogFunc env
                  => StackYamlLoc (Path Abs File)
                  -- ^ Override stack.yaml
-                 -> m (LocalConfigStatus (Path Abs File))
+                 -> RIO env (LocalConfigStatus (Path Abs File))
 getProjectConfig (SYLOverride stackYaml) = return $ LCSProject stackYaml
 getProjectConfig SYLDefault = do
     env <- liftIO getEnvironment
@@ -839,7 +847,7 @@ getProjectConfig SYLDefault = do
     getStackDotYaml dir = do
         let fp = dir </> stackDotYaml
             fp' = toFilePath fp
-        logDebug $ "Checking for project config at: " <> T.pack fp'
+        logDebug $ "Checking for project config at: " <> fromString fp'
         exists <- doesFileExist fp
         if exists
             then return $ Just fp
@@ -856,17 +864,17 @@ data LocalConfigStatus a
 -- | Find the project config file location, respecting environment variables
 -- and otherwise traversing parents. If no config is found, we supply a default
 -- based on current directory.
-loadProjectConfig :: (MonadIO m, MonadThrow m, MonadLogger m)
+loadProjectConfig :: HasLogFunc env
                   => StackYamlLoc (Path Abs File)
                   -- ^ Override stack.yaml
-                  -> m (LocalConfigStatus (Project, Path Abs File, ConfigMonoid))
+                  -> RIO env (LocalConfigStatus (Project, Path Abs File, ConfigMonoid))
 loadProjectConfig mstackYaml = do
     mfp <- getProjectConfig mstackYaml
     case mfp of
         LCSProject fp -> do
             currDir <- getCurrentDir
             logDebug $ "Loading project config file " <>
-                        T.pack (maybe (toFilePath fp) toFilePath (stripProperPrefix currDir fp))
+                        fromString (maybe (toFilePath fp) toFilePath (stripProperPrefix currDir fp))
             LCSProject <$> load fp
         LCSNoProject -> do
             logDebug "No project config file found, using defaults."
@@ -883,8 +891,8 @@ loadProjectConfig mstackYaml = do
 -- If a file already exists at the deprecated location, its location is returned.
 -- Otherwise, the new location is returned.
 getDefaultGlobalConfigPath
-    :: (MonadIO m, MonadLogger m)
-    => m (Maybe (Path Abs File))
+    :: HasLogFunc env
+    => RIO env (Maybe (Path Abs File))
 getDefaultGlobalConfigPath =
     case (defaultGlobalConfigPath, defaultGlobalConfigPathDeprecated) of
         (Just new,Just old) ->
@@ -901,8 +909,8 @@ getDefaultGlobalConfigPath =
 -- If a file already exists at the deprecated location, its location is returned.
 -- Otherwise, the new location is returned.
 getDefaultUserConfigPath
-    :: (MonadIO m, MonadLogger m)
-    => Path Abs Dir -> m (Path Abs File)
+    :: HasLogFunc env
+    => Path Abs Dir -> RIO env (Path Abs File)
 getDefaultUserConfigPath stackRoot = do
     (path, exists) <- tryDeprecatedPath
         (Just "non-project configuration file")
