@@ -23,6 +23,7 @@ module Stack.GhcPkg
 
 import           Stack.Prelude
 import qualified Data.ByteString.Char8 as S8
+import qualified Data.ByteString.Lazy as BL
 import           Data.List
 import qualified Data.Text as T
 import qualified Data.Text.Encoding as T
@@ -37,16 +38,16 @@ import           Stack.Types.Compiler
 import           Stack.Types.PackageName
 import           Stack.Types.Version
 import           System.FilePath (searchPathSeparator)
-import           System.Process.Read
+import           RIO.Process
 
 -- | Get the global package database
-getGlobalDB :: (MonadUnliftIO m, MonadLogger m)
-            => EnvOverride -> WhichCompiler -> m (Path Abs Dir)
-getGlobalDB menv wc = do
+getGlobalDB :: (HasProcessContext env, HasLogFunc env)
+            => WhichCompiler -> RIO env (Path Abs Dir)
+getGlobalDB wc = do
     logDebug "Getting global package database location"
     -- This seems like a strange way to get the global package database
     -- location, but I don't know of a better one
-    bs <- ghcPkg menv wc [] ["list", "--global"] >>= either throwIO return
+    bs <- ghcPkg wc [] ["list", "--global"] >>= either throwIO return
     let fp = S8.unpack $ stripTrailingColon $ firstLine bs
     liftIO $ resolveDir' fp
   where
@@ -57,27 +58,29 @@ getGlobalDB menv wc = do
     firstLine = S8.takeWhile (\c -> c /= '\r' && c /= '\n')
 
 -- | Run the ghc-pkg executable
-ghcPkg :: (MonadUnliftIO m, MonadLogger m)
-       => EnvOverride
-       -> WhichCompiler
+ghcPkg :: (HasProcessContext env, HasLogFunc env)
+       => WhichCompiler
        -> [Path Abs Dir]
        -> [String]
-       -> m (Either ReadProcessException S8.ByteString)
-ghcPkg menv wc pkgDbs args = do
+       -> RIO env (Either SomeException S8.ByteString)
+ghcPkg wc pkgDbs args = do
     eres <- go
     case eres of
-          Left _ -> do
-              mapM_ (createDatabase menv wc) pkgDbs
-              go
-          Right _ -> return eres
+      Left _ -> do
+        mapM_ (createDatabase wc) pkgDbs
+        go
+      Right _ -> return eres
   where
-    go = tryProcessStdout Nothing menv (ghcPkgExeName wc) args'
+    go = fmap (fmap BL.toStrict)
+       $ tryAny
+       $ proc (ghcPkgExeName wc) args' readProcessStdout_
     args' = packageDbFlags pkgDbs ++ args
 
 -- | Create a package database in the given directory, if it doesn't exist.
-createDatabase :: (MonadUnliftIO m, MonadLogger m)
-               => EnvOverride -> WhichCompiler -> Path Abs Dir -> m ()
-createDatabase menv wc db = do
+createDatabase
+  :: (HasProcessContext env, HasLogFunc env)
+  => WhichCompiler -> Path Abs Dir -> RIO env ()
+createDatabase wc db = do
     exists <- doesFileExist (db </> $(mkRelFile "package.cache"))
     unless exists $ do
         -- ghc-pkg requires that the database directory does not exist
@@ -86,11 +89,10 @@ createDatabase menv wc db = do
         dirExists <- doesDirExist db
         args <- if dirExists
             then do
-                logWarn $ T.pack $ concat
-                    [ "The package database located at "
-                    , toFilePath db
-                    , " is corrupted (missing its package.cache file)."
-                    ]
+                logWarn $
+                    "The package database located at " <>
+                    fromString (toFilePath db) <>
+                    " is corrupted (missing its package.cache file)."
                 logWarn "Proceeding with a recache"
                 return ["--package-db", toFilePath db, "recache"]
             else do
@@ -99,12 +101,9 @@ createDatabase menv wc db = do
                 -- finding out it isn't the hard way
                 ensureDir (parent db)
                 return ["init", toFilePath db]
-        eres <- tryProcessStdout Nothing menv (ghcPkgExeName wc) args
-        case eres of
-            Left e -> do
-                logError $ T.pack $ "Unable to create package database at " ++ toFilePath db
-                throwIO e
-            Right _ -> return ()
+        void $ proc (ghcPkgExeName wc) args $ \pc ->
+          readProcessStdout_ pc `onException`
+          logError ("Unable to create package database at " <> fromString (toFilePath db))
 
 -- | Get the name to use for "ghc-pkg", given the compiler version.
 ghcPkgExeName :: WhichCompiler -> String
@@ -124,51 +123,47 @@ packageDbFlags pkgDbs =
 
 -- | Get the value of a field of the package.
 findGhcPkgField
-    :: (MonadUnliftIO m, MonadLogger m)
-    => EnvOverride
-    -> WhichCompiler
+    :: (HasProcessContext env, HasLogFunc env)
+    => WhichCompiler
     -> [Path Abs Dir] -- ^ package databases
     -> String -- ^ package identifier, or GhcPkgId
     -> Text
-    -> m (Maybe Text)
-findGhcPkgField menv wc pkgDbs name field = do
+    -> RIO env (Maybe Text)
+findGhcPkgField wc pkgDbs name field = do
     result <-
         ghcPkg
-            menv
             wc
             pkgDbs
             ["field", "--simple-output", name, T.unpack field]
     return $
         case result of
             Left{} -> Nothing
-            Right lbs ->
-                fmap (stripCR . T.decodeUtf8) $ listToMaybe $ S8.lines lbs
+            Right bs ->
+                fmap (stripCR . T.decodeUtf8) $ listToMaybe $ S8.lines bs
 
 -- | Get the version of the package
-findGhcPkgVersion :: (MonadUnliftIO m, MonadLogger m)
-                  => EnvOverride
-                  -> WhichCompiler
+findGhcPkgVersion :: (HasProcessContext env, HasLogFunc env)
+                  => WhichCompiler
                   -> [Path Abs Dir] -- ^ package databases
                   -> PackageName
-                  -> m (Maybe Version)
-findGhcPkgVersion menv wc pkgDbs name = do
-    mv <- findGhcPkgField menv wc pkgDbs (packageNameString name) "version"
+                  -> RIO env (Maybe Version)
+findGhcPkgVersion wc pkgDbs name = do
+    mv <- findGhcPkgField wc pkgDbs (packageNameString name) "version"
     case mv of
         Just !v -> return (parseVersion v)
         _ -> return Nothing
 
-unregisterGhcPkgId :: (MonadUnliftIO m, MonadLogger m)
-                    => EnvOverride
-                    -> WhichCompiler
+unregisterGhcPkgId :: (HasProcessContext env, HasLogFunc env)
+                    => WhichCompiler
                     -> CompilerVersion 'CVActual
                     -> Path Abs Dir -- ^ package database
                     -> GhcPkgId
                     -> PackageIdentifier
-                    -> m ()
-unregisterGhcPkgId menv wc cv pkgDb gid ident = do
-    eres <- ghcPkg menv wc [pkgDb] args
+                    -> RIO env ()
+unregisterGhcPkgId wc cv pkgDb gid ident = do
+    eres <- ghcPkg wc [pkgDb] args
     case eres of
-        Left e -> logWarn $ T.pack $ show e
+        Left e -> logWarn $ displayShow e
         Right _ -> return ()
   where
     -- TODO ideally we'd tell ghc-pkg a GhcPkgId instead
@@ -179,12 +174,11 @@ unregisterGhcPkgId menv wc cv pkgDb gid ident = do
             _ -> ["--ipid", ghcPkgIdString gid])
 
 -- | Get the version of Cabal from the global package database.
-getCabalPkgVer :: (MonadUnliftIO m, MonadLogger m)
-               => EnvOverride -> WhichCompiler -> m Version
-getCabalPkgVer menv wc = do
+getCabalPkgVer :: (HasProcessContext env, HasLogFunc env)
+               => WhichCompiler -> RIO env Version
+getCabalPkgVer wc = do
     logDebug "Getting Cabal package version"
     mres <- findGhcPkgVersion
-        menv
         wc
         [] -- global DB
         cabalPackageName
