@@ -64,6 +64,8 @@ data BuildOpts =
             -- ^ Watch files for changes and automatically rebuild
             ,boptsKeepGoing :: !(Maybe Bool)
             -- ^ Keep building/running after failure
+            ,boptsKeepTmpFiles :: !(Maybe Bool)
+            -- ^ Keep intermediate files and build directories
             ,boptsForceDirty :: !Bool
             -- ^ Force treating all local packages as having dirty files
 
@@ -105,6 +107,7 @@ defaultBuildOpts = BuildOpts
     , boptsInstallCompilerTool = False
     , boptsPreFetch = False
     , boptsKeepGoing = Nothing
+    , boptsKeepTmpFiles = Nothing
     , boptsForceDirty = False
     , boptsTests = False
     , boptsTestOpts = defaultTestOpts
@@ -172,6 +175,7 @@ data BuildOptsMonoid = BuildOptsMonoid
     , buildMonoidInstallCompilerTool :: !(First Bool)
     , buildMonoidPreFetch :: !(First Bool)
     , buildMonoidKeepGoing :: !(First Bool)
+    , buildMonoidKeepTmpFiles :: !(First Bool)
     , buildMonoidForceDirty :: !(First Bool)
     , buildMonoidTests :: !(First Bool)
     , buildMonoidTestOpts :: !TestOptsMonoid
@@ -202,6 +206,7 @@ instance FromJSON (WithJSONWarnings BuildOptsMonoid) where
               buildMonoidInstallCompilerTool <- First <$> o ..:? buildMonoidInstallCompilerToolArgName
               buildMonoidPreFetch <- First <$> o ..:? buildMonoidPreFetchArgName
               buildMonoidKeepGoing <- First <$> o ..:? buildMonoidKeepGoingArgName
+              buildMonoidKeepTmpFiles <- First <$> o ..:? buildMonoidKeepTmpFilesArgName
               buildMonoidForceDirty <- First <$> o ..:? buildMonoidForceDirtyArgName
               buildMonoidTests <- First <$> o ..:? buildMonoidTestsArgName
               buildMonoidTestOpts <- jsonSubWarnings (o ..:? buildMonoidTestOptsArgName ..!= mempty)
@@ -255,6 +260,9 @@ buildMonoidPreFetchArgName = "prefetch"
 buildMonoidKeepGoingArgName :: Text
 buildMonoidKeepGoingArgName = "keep-going"
 
+buildMonoidKeepTmpFilesArgName :: Text
+buildMonoidKeepTmpFilesArgName = "keep-tmp-files"
+
 buildMonoidForceDirtyArgName :: Text
 buildMonoidForceDirtyArgName = "force-dirty"
 
@@ -282,9 +290,12 @@ buildMonoidSplitObjsName = "split-objs"
 buildMonoidSkipComponentsName :: Text
 buildMonoidSkipComponentsName = "skip-components"
 
+instance Semigroup BuildOptsMonoid where
+    (<>) = mappenddefault
+
 instance Monoid BuildOptsMonoid where
     mempty = memptydefault
-    mappend = mappenddefault
+    mappend = (<>)
 
 -- | Which subset of packages to build
 data BuildSubset
@@ -339,9 +350,12 @@ toMonoidCoverageArgName = "coverage"
 toMonoidDisableRunArgName :: Text
 toMonoidDisableRunArgName = "no-run-tests"
 
+instance Semigroup TestOptsMonoid where
+  (<>) = mappenddefault
+
 instance Monoid TestOptsMonoid where
   mempty = memptydefault
-  mappend = mappenddefault
+  mappend = (<>)
 
 
 
@@ -362,9 +376,12 @@ instance FromJSON (WithJSONWarnings HaddockOptsMonoid) where
     (\o -> do hoMonoidAdditionalArgs <- o ..:? hoMonoidAdditionalArgsName ..!= []
               return HaddockOptsMonoid{..})
 
+instance Semigroup HaddockOptsMonoid where
+  (<>) = mappenddefault
+
 instance Monoid HaddockOptsMonoid where
   mempty = memptydefault
-  mappend = mappenddefault
+  mappend = (<>)
 
 hoMonoidAdditionalArgsName :: Text
 hoMonoidAdditionalArgsName = "haddock-args"
@@ -401,9 +418,12 @@ beoMonoidAdditionalArgsArgName = "benchmark-arguments"
 beoMonoidDisableRunArgName :: Text
 beoMonoidDisableRunArgName = "no-run-benchmarks"
 
+instance Semigroup BenchmarkOptsMonoid where
+  (<>) = mappenddefault
+
 instance Monoid BenchmarkOptsMonoid where
   mempty = memptydefault
-  mappend = mappenddefault
+  mappend = (<>)
 
 data FileWatchOpts
   = NoFileWatch
