@@ -9,7 +9,7 @@
 
 module Stack.Types.Docker where
 
-import Stack.Prelude
+import Stack.Prelude hiding (Display (..))
 import Data.Aeson.Extended
 import Data.List (intercalate)
 import qualified Data.Text as T
@@ -106,11 +106,11 @@ data DockerOptsMonoid = DockerOptsMonoid
 -- | Decode uninterpreted docker options from JSON/YAML.
 instance FromJSON (WithJSONWarnings DockerOptsMonoid) where
   parseJSON = withObjectWarnings "DockerOptsMonoid"
-    (\o -> do dockerMonoidDefaultEnable    <- pure (Any True)
+    (\o -> do let dockerMonoidDefaultEnable = Any True
               dockerMonoidEnable           <- First <$> o ..:? dockerEnableArgName
               dockerMonoidRepoOrImage      <- First <$>
-                                              (((Just . DockerMonoidImage) <$> o ..: dockerImageArgName) <|>
-                                              ((Just . DockerMonoidRepo) <$> o ..: dockerRepoArgName) <|>
+                                              ((Just . DockerMonoidImage <$> o ..: dockerImageArgName) <|>
+                                              (Just . DockerMonoidRepo <$> o ..: dockerRepoArgName) <|>
                                               pure Nothing)
               dockerMonoidRegistryLogin    <- First <$> o ..:? dockerRegistryLoginArgName
               dockerMonoidRegistryUsername <- First <$> o ..:? dockerRegistryUsernameArgName
@@ -131,10 +131,14 @@ instance FromJSON (WithJSONWarnings DockerOptsMonoid) where
                                                    ..!= VersionRangeJSON anyVersion)
               return DockerOptsMonoid{..})
 
+-- | Left-biased combine Docker options
+instance Semigroup DockerOptsMonoid where
+  (<>) = mappenddefault
+
 -- | Left-biased combine Docker options
 instance Monoid DockerOptsMonoid where
   mempty = memptydefault
-  mappend = mappenddefault
+  mappend = (<>)
 
 -- | Where to get the `stack` executable to run in Docker containers
 data DockerStackExe
