@@ -6,7 +6,7 @@ module Stack.Build.TargetSpec (main, spec) where
 import qualified Data.Text           as T
 import           Stack.Build.Target
 import           Stack.Prelude
-import           Stack.Types.Config
+import           Stack.Types.NamedComponent
 import           Stack.Types.PackageIdentifier
 import           Stack.Types.PackageName
 import           Stack.Types.Version
